/*
SerialDisplay.cpp - Library for Sparkfunk serialdisplay

by Cordier Yves Jan 2015 Version du 01/02/2015
modifiée le 29/05/2015

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/******************************************************************************
* Includes
******************************************************************************/
#include "SerialDisplay.h"


boolean Aff_Serial_deja_calcule=false ; // on a deja evalué le temps mis pour effectuer une 
// opération du style digitalWrite();
long Aff_Serial_delay_out=7;  // le temps en micro secondes pour cette operation valeur par defaut

// implementation de l'objet SerialOut
// Le constructeur par defaut
SerialOut::SerialOut()
{}

// Le constructeur alternatif
SerialOut::SerialOut(byte tx)

{
	_transmitPin=tx;
	_baudRate = 9600;
	
	pinMode(_transmitPin,OUTPUT);
}

// Reglage des parametres de transmission
void SerialOut::setSerial(long speed)
{
	_baudRate = speed;
	_bitPeriod = 1000000 / _baudRate; // en microsecondes
	if (! Aff_Serial_deja_calcule)
	{
		long T2;
		boolean reste=true;
		// recherche du temps pour afficher une sortie
		while (reste) // pour eviter le bouclage de la fonction micros()
		{
			
			long T1=micros();
			
			for (int i=0; i<1000; i++)
			{
				digitalWrite(_transmitPin,LOW);
			}
			T2=micros();
			reste=T2<T1;
			T2=T2-T1;
		}
		
		Aff_Serial_delay_out=T2/1000; // temps pour afficher une sortie
		Aff_Serial_deja_calcule=true;
	}
	digitalWrite(_transmitPin, HIGH);
	delayMicroseconds( 2*_bitPeriod); // if we were low this establishes the end
}

// Initialisation de la sortie serie
void SerialOut::begin(long vitesse)
{ 
	setSerial(vitesse); 
}

// La clef de voute du systeme : ecrire un caractère sur la sortie serie
void SerialOut::write(char b)
{
	if (_baudRate == 0)
	return;  // par securité

	long bitDelay = _bitPeriod -Aff_Serial_delay_out; // calcul le temps reel entre deux impulsions
	uint8_t oldSREG = SREG; // On sauve le registre d'interruption
    cli(); // masquage des interruptions
	digitalWrite(_transmitPin, LOW); // initialisation de la transmission
	delayMicroseconds(bitDelay);
	// Serialisation des bits du caractère (8 bits) à transmettre)
	for (byte mask = 0x01; mask; mask <<= 1) 
	{
		digitalWrite(_transmitPin,(byte(b) & mask));
		delayMicroseconds(bitDelay);
	}

	digitalWrite(_transmitPin, HIGH); // Le stop Bit
	delayMicroseconds(2*bitDelay); // par securité on attends qq microsecondes de plus
	SREG=oldSREG; // on remet en place le registre d'interruption
	
}

// | Divers constantes pour gérer l'afficheur serie|
// =================================================
#define Ctrl_j 10
#define Ctrl_k 11
#define Ctrl_l 12
#define Ctrl_m 13 // code pour passe à 9600 bauds
#define Ctrl_n 14
#define Ctrl_o 15
#define Ctrl_p 16
#define Ctrl_q 17
#define Ctrl_r 18
#define bauds_9600 Ctrl_m
#define Car_cmd 254
#define Car_cmd_special 124
#define Clear_lcd 01
#define CursRight_lcd 0x14
#define CursLeft_lcd 0x10
#define ScrollRight_lcd 0x1C
#define ScrollLeft_lcd 0x18
#define Displayon_lcd 0x0C
#define Displayoff_lcd 0x08
#define Underlineon_lcd 0x0E
#define Underlineoff_lcd 0x0C
#define Blinkon_lcd 0x0D
#define Blinkoff_lcd 0x0F // a verifier

// les vitesses de transmission
long SerialDisplay_vitesse_afficheur [6] = {
	2400, 4800, 9600, 14400, 19200, 38400
};
// byte commande_vitesse_afficheur[6] = {Ctrl_k, Ctrl_l, Ctrl_m, Ctrl_n, Ctrl_o, Ctrl_p};



// Le constructeur
SerialDisplay::SerialDisplay(byte tx)

{	_transmitPin=tx;
	_baudRate = 9600;
	
	pinMode(_transmitPin,OUTPUT);
	
	_Max_X=Max_X;
	_Max_Y=Max_Y;
	
}
// Le constructeur
SerialDisplay::SerialDisplay(byte tx, byte sizex, byte sizey)

{
	cursX=0;
	_Max_X=sizex;
	_Max_Y=sizey;
	if ((_Max_X!=16) &(_Max_X!=20)) {_Max_X=16;}; // recadrage en cas de valeurs fantaisistes
	if ((_Max_Y!=2) & (_Max_Y!=4)) {_Max_Y=2;}; // recadrage en cas de valeurs fantaisistes
	
}

void SerialDisplay::Clrscr()
{
	write(char(Car_cmd));
	write(char(Clear_lcd));
	cursX=0;
	
}

void SerialDisplay::set_size_lcd()
{
	write(Car_cmd_special);
	if (_Max_X==20) {write(3);} // 20 caracteres par ligne
	else {write(4);}; // par defaut 18
	delay(100);
	write(Car_cmd_special);
	if (_Max_Y==4) {write(5);} // 4 lignes de texte
	else {write(6);}; // par defaut 2
	delay(100); // pour pouvoir ecrire en mémoire EEPROM sans risques
}
// Gestion des curseurs sur l'afficheur
// ************************************
// positionne le curseur à x,y (le point en haut à gauche à comme coordonnées (1,1)

void SerialDisplay::Posxy(byte x, byte y)
{ 
	byte code = 128; // bit de poids fort à 1
	y=y % (_Max_Y+1); // enroulement des lignes en fonction du nombre de lignes
	x=x % (_Max_X+1); //  enroulement des colonnes en fonction du nombre de colonnes
	if (_Max_X=16) // cas 16 colonnes
	{
		switch (y)
		{
		case 1 : {code =code+(x-1); break;}
		case 2 : {code =code+64+(x-1); break;}
		case 3 : {code =code+16+(x-1); break;}
		case 4 : {code =code+80+(x-1); break;}
		}
	}
	else // cas 20 colonnes
	{	switch (y)
		{
		case 1 : {code =code+(x-1); break;}
		case 2 : {code =code+64+(x-1); break;}
		case 3 : {code =code+20+(x-1); break;}
		case 4 : {code =code+84+(x-1); break;}
		}
	}
	write(char(Car_cmd));
	write(char(code));
	cursX=x-1; // mise à jour du curseur logiciel
}


// Initialisation de l'afficheur
// *****************************
void SerialDisplay::begin(long vitesse)
{  
    // SerialOut:begin(vitesse);
	setSerial(vitesse); // Calcul des constantes de transmission si besoin
	// Tentative au boot de l'afficheur de le mettre é 9600 bauds
	setSerial(9600); 
	write(char(Ctrl_r));
	
	// On scan tous les modes possibles pour le remettre à 9600 bauds
	for (int i = 0; i < 6; i++)
	{
		setSerial(SerialDisplay_vitesse_afficheur[i]);
		delay(100);
		write(char(Car_cmd_special));
		write(char(Ctrl_m) ); // Reset l'afficheur à 9600
		
	}
	setSerial(9600); // Remise de la serie en 9600 bauds
	delay(100);
	
	// On bascule l'afficheur à la vitesse demandée
	for (int i = 0; i < 6; i++)
	{ if (SerialDisplay_vitesse_afficheur[i] == vitesse)
		{
			// Commande de l'afficheur
			write(char(Car_cmd_special));
			write(char(Ctrl_k+i));
			// On bascule la série au bon mode
			setSerial(vitesse);
			delay(100);
		}
	}
	set_size_lcd();	
	Clrscr();
	delay(100);
	Backlight(30);
	delay(100);
	println(F("Display OK"));
	print(_baudRate);
	print (F(" bds"));
	delay(3000);
	Clrscr();
}
void SerialDisplay::CursRight()
{
	write(char(Car_cmd));
	write(char(CursRight_lcd));
	cursX=cursX++ % _Max_X;
}
void SerialDisplay::CursLeft()
{
	write(char(Car_cmd));
	write(char(CursLeft_lcd));
	cursX= cursX-- % _Max_X;
}
void SerialDisplay::ScrollRight()
{
	write(char(Car_cmd));
	write(char(ScrollRight_lcd));
}
void SerialDisplay::ScrollLeft()
{
	write(char(Car_cmd));
	write(char(ScrollLeft_lcd));
};
void SerialDisplay::Displayon()
{
	write(char(Car_cmd));
	write(char(Displayon_lcd));
}
void SerialDisplay::Displayoff()
{
	write(char(Car_cmd));
	write(char(Displayoff_lcd));
}
void SerialDisplay::Underlineon()
{
	write(char(Car_cmd));
	write(char(Underlineon_lcd));
}
void SerialDisplay::Underlineoff()
{
	write(char(Car_cmd));
	write(char(Underlineoff_lcd));
}
void SerialDisplay::Blinkingon()
{
	write(char(Car_cmd));
	write(char(Blinkon_lcd));
}
void SerialDisplay::Blinkingoff()
{
	write(char(Car_cmd));
	write(char(Blinkoff_lcd));
}
void SerialDisplay::Backlight(byte i)
{
	if (i>30) {i=30;}
	write(char(124));
	write(char(127+i));
}

void SerialDisplay::print(const char *s)
{ 
	while (*s)
	{
		print(*s++); // imprime le caractère en mode caractère
	}
}

void SerialDisplay::print(String s)
{ int i=0;
	while (s[i])
	{
		print(s[i]);  // imprime le caractère en mode caractère
		i++; //passe au caractere suivant
	}
}


void SerialDisplay::print(byte b)
{
	print((long) b);
}


void SerialDisplay::print(char c)
{ 
	if ((c>=char(32))&(c<=char(126))& (c!=char(124))) 
	{write(c);}
	else 
	{write(char(32));}
	cursX++;
	cursX= cursX % _Max_X; // avance le curseur logiciel de 1
}

void SerialDisplay::print(int n)
{
	print((long) n);
}

void SerialDisplay::print(unsigned int n)
{
	print((unsigned long) n);
}

void SerialDisplay::print(long n)
{
	if (n < 0) {
		print('-');
		n = -n;
	}
	printNumber(n, 10);
}

void SerialDisplay::print(unsigned long n)
{
	printNumber(n, 10);
}

void SerialDisplay::print(long n, int base)
{
	if (base == 0)
	print((char) n);
	else if (base == 10)
	print(n);
	else
	printNumber(n, base);
}

void SerialDisplay::print(double n, int d)
{
	printFloat(n, d);
}

void SerialDisplay::print(double n)
{
	printFloat(n, 2);
}

void SerialDisplay::print(float n, int d)
{
	printFloat(n, d);
}

void SerialDisplay::print(float n)
{
	printFloat(n, 2);
}

void SerialDisplay::println(void)
{
	while (cursX!=0){print(char(32));}; // On complete la ligne de blanc et on passe à la ligne automatiquement
	
}

void SerialDisplay::println(char c)
{
	print(c);
	println();  
}

void SerialDisplay::println(const char c[])
{
	print(c);
	println();
}

void SerialDisplay::println(String s)
{
	print(s);
	println();
}

void SerialDisplay::println(byte b)
{
	print(b);
	println();
}

void SerialDisplay::println(int n)
{
	print(n);
	println();
}

void SerialDisplay::println(long n)
{
	print(n);
	println();  
}

void SerialDisplay::println(unsigned long n)
{
	print(n);
	println();  
}

void SerialDisplay::println(long n, int base)
{
	print(n, base);
	println();
}

void SerialDisplay::println(double n, int d)
{
	print(n, d);
	println();
}

void SerialDisplay::println(double n)
{
	print(n, 2);
	println();
}

void SerialDisplay::println(float n, int d)
{
	print(n, d);
	println();
}

void SerialDisplay::println(float n)
{
	print(n, 2);
	println();
}
// Private Methods /////////////////////////////////////////////////////////////

void SerialDisplay::printNumber(unsigned long n, uint8_t base)
{
	unsigned char buf[8 * sizeof(long)]; // Assumes 8-bit chars.
	unsigned long i = 0;

	if (n == 0) {
		print('0');
		return;
	}

	while (n > 0) {
		buf[i++] = n % base;
		n /= base;
	}

	for (; i > 0; i--)
	print((char) (buf[i - 1] < 10 ? '0' + buf[i - 1] : 'A' + buf[i - 1] - 10));
	/* Forme condensée de
	for(;i>0;i--)
	{if (buf[i-1]<10)
		{
		print (char(buf[i-1]+48)); // 48 code de 0
		}
		else
		{
		print (char(buf[i-1]-10+65)); // 65 code de A pour affichage en hexa
		}
	}
	
	*/
	
}

void SerialDisplay::printFloat(double number, uint8_t digits)
{
	// Handle negative numbers
	if (number < 0.0)
	{
		print('-');
		number = -number;
	}

	// Round correctly so that print(1.999, 2) prints as "2.00"
	double rounding = 0.5;
	for (uint8_t i=0; i<digits; ++i) // compute the smallest part of rounding depends on digis
	rounding /= 10.0;

	number += rounding; // increase the number by this small part

	// Extract the integer part of the number and print it
	unsigned long int_part = (unsigned long)number;
	double remainder = number - (double)int_part;
	print(int_part);

	// Print the decimal point, but only if there are digits beyond
	if (digits > 0)
	print('.');

	// Extract digits from the remainder one at a time
	while (digits-- > 0)
	{
		remainder *= 10.0;
		int toPrint = int(remainder);
		print(toPrint);
		remainder -= toPrint;
	}
}
