/*
SerialDisplay.h - Library for Sparkfunk serialdisplay

by Cordier Yves Jan 2015 Version du 01/02/2015

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
/* Cette librairie permet l'utilisation de sorties séries sur une broche
   par l'utilisation d'un objert de type SerialOut
   exemple
   SerialOut ma_serie(3); // ouvre une sortie serie sur la broche 3
   ..
   ma_serie.begin(9600); // regle la vitesse de transmission à 9600 bauds 
                         // 300, 600, 1200,2400, 4800, 9600, 14400, (19200 38400 non testé)

   ma_serie.write(10); // envoi le caractère 10 0x0A au sérialisateur
 */

#ifndef SerialDisplay_h
#define SerialDisplay_h

#if defined(ARDUINO) && ARDUINO >= 100
#include "Arduino.h"
#else
#include "WProgram.h"
#endif

#include <inttypes.h>
#include <avr/interrupt.h>

#define Max_X 16 // Nb de caractere par lignes 16 ou 20
#define Max_Y 2 // Nb de lignes de caracterees 4 ou 2



class SerialOut
{
// private:
protected: // ces attributs seront visibles dans les classes derivés mais non dans le pgm principal
	uint8_t _transmitPin;
	long _baudRate ;
	int _bitPeriod;
	void setSerial(long speed); // mise en place des constantes de transmission en fonction de la vitesse

 // le delay en micro secondes pour une sortie sur broche
	
public:
// Les constructeurs
	SerialOut(); // Le constructeur par defaut
	SerialOut(uint8_t); // defini la broche de connection de la sortie serie
	
	void begin(long); // defini la vitesse de communication

	void write(char); // la clef de voute du systeme envoi un caractère sur la voie série
	
};



class SerialDisplay : public SerialOut
{
private:
	/*uint8_t _transmitPin;
	long _baudRate ;
	int _bitPeriod;
	void setSerial(long speed); // mise en place des constantes de transmission en fonction de la vitesse
*/
	void printNumber(unsigned long, uint8_t);
	int cursX; // Le curseur logiciel cursX de 0.. _Max_X-1
	// position haut gauche (0,0)
	int _Max_X ; // Nb de caractères sur x
	int _Max_Y ; // Le nombre de lignes
	void printFloat(double , uint8_t );
	void set_size_lcd();
public:
// Les constructeurs
	SerialDisplay(byte tx); // defini la broche de connection de l'afficheur par defaut un afficheur de 16*2
	SerialDisplay(uint8_t,uint8_t,uint8_t);// defini la broche de connection de l'afficheur ainsi que la taille de l'afficheur
	                                        // broche, tx , ty ; tx vaut 16 ou 20 ty vaut 2 ou 4
	void begin(long vitesse); 
	// void write(char); // la clef de voute du systeme envoi un caractère sur la voie série
	void print(char);
	void print(byte);
	void print(const char[]);
	void print(String);
	void print(int);
	void print(unsigned int);
	void print(long);
	void print(unsigned long);
	void print(long, int); // impression dans une base donnée int 0..16
	void print(double); // impression avec 2 chiffres derriere la virgule
	void print(double , int); // impression avec int digits derriere la virgule
	void print(float); // impression avec 2 chiffres derriere la virgule
	void print(float,int);// impression avec int digits derriere la virgule
	
	
	// les memes procedures avec passage à la ligne
	void println(char);
	void println(byte);
	void println(const char[]);
	void println(String);
	void println(int);
	void println(unsigned int);
	void println(long);
	void println(unsigned long);
	void println(long, int);
	void println(double , int);
	void println(double );
	void println(float);
	void println(float,int);
	void println(void);
	
	void Clrscr(); //efface l'écran
	void CursRight(); //bouge le curseur vers la droite
	void CursLeft();  //bouge le curseur vers la gauche
	void ScrollRight(); //bouge tout l'écran vers la droite
	void ScrollLeft(); //bouge tout l'écran vers la gauche
	void Displayon(); //affiche on
	void Displayoff(); //affiche off
	void Underlineon(); //souligne le curseur
	void Underlineoff(); //curseur muet
	void Blinkingon(); //fait clignotter le curseur
	void Blinkingoff(); //arrete le clignottement
	void Posxy(byte x, byte y); //positionne le curseur en x y (le coin en haut à gauche est à (1,1)
	void Backlight(byte i); //fait varier la puissance lumineuse i va de 1 à 31
	
};

#endif

