/*
Ultrasonic.h - Library for HR-SC04 Ultrasonic Ranging Module.
Created by ITead studio. Alex, Apr 20, 2010.
iteadstudio.com

updated by noonv. Feb, 2011
http://robocraft.ru
modified by Cordier Yves Jan 2015
*/

#ifndef Ultrasonic_h
#define Ultrasonic_h

#if defined(ARDUINO) && ARDUINO >= 100
#include "Arduino.h"
#else
#include "WProgram.h"
#endif

// Pour la sortie l'unité
#define Ultrasonic_unite_CM 1
#define CM 1
#define Ultrasonic_unite_INC 0
#define INC 0

// Les erreurs codifiées
#define Ultrasonic_out_of_limit  1
#define Ultrasonic_out_of_delay 2
#define Ultrasonic_no_error 0
#define Ultrasonic_delay_between_two_salves 60000 // comme son nom l'indique

class Ultrasonic
{
public:

	Ultrasonic(int TP, int EP); // Le createur
	long Timing(); // effectue la mesure et renvoi la longueur impulsion en micro-secondes
	float Ranging(int unites); // effectue la mesure et renvoi le resultat de la mesure dans l'unité choisie
	int Error(); // renvoi le code d'erreur de la dernière mesure
	unsigned long Last_call(); // renvoi en milisecondes l'instant de la dernière mesure
	
private:
	int Trig_pin;
	int Echo_pin;
	long  duration;
	float distance_cm ,distance_inc;
	unsigned long m_Last_call;
	int m_error;
	unsigned long delay_between(unsigned long T2, unsigned long T1); // Calcul un delai en micro_secondes

};

#endif //#ifndef Ultrasonic_h
